@@ -97,10 +97,14 @@ def force_unicode(value):
         # Python 3.X
         if isinstance(value, bytes):
             value = value.decode('utf-8', errors='replace')
+        elif not isinstance(value, str):
+            value = str(value)
     else:
         # Python 2.X
         if isinstance(value, str):
             value = value.decode('utf-8', errors='replace')
+        elif not isinstance(value, basestring):
+            value = unicode(value)
 
     return value
 
@@ -233,13 +237,15 @@ class Solr(object):
         # No path? No problem.
         return self.url
 
-    def _send_request(self, method, path='', body=None, headers=None):
+    def _send_request(self, method, path='', body=None, headers=None, files=None):
         url = self._create_full_url(path)
         method = method.lower()
         log_body = body
 
         if log_body is None:
             log_body = ''
+        elif not isinstance(log_body, str):
+            log_body = repr(body)
 
         self.log.debug("Starting request to '%s' (%s) with body '%s'...",
                        url, method, log_body[:10])
@@ -264,7 +270,8 @@ class Solr(object):
                 for k, v in headers.items():
                     bytes_headers[force_bytes(k)] = force_bytes(v)
 
-            resp = requests_method(url, data=bytes_body, headers=bytes_headers, timeout=self.timeout)
+            resp = requests_method(url, data=bytes_body, headers=bytes_headers, files=files,
+                                   timeout=self.timeout)
         except requests.exceptions.Timeout as err:
             error_message = "Connection to server '%s' timed out: %s"
             self.log.error(error_message, [url, err], exc_info=True)
@@ -670,34 +677,24 @@ class Solr(object):
                 doc_elem.set('boost', force_unicode(value))
                 continue
 
-            # handle lists & tuples
+            # To avoid multiple code-paths we'd like to treat all of our values as iterables:
             if isinstance(value, (list, tuple)):
-                for bit in value:
-                    if self._is_null_value(value):
-                        continue
-
-                    if boost and bit in boost:
-                        boost[bit] = force_unicode(boost[bit])
-
-                        field = ET.Element('field', name=key, boost=boost[bit])
-                    else:
-                        field = ET.Element('field', name=key)
-
-                    field.text = self._from_python(bit)
-                    doc_elem.append(field)
-            # handle strings and unicode
+                values = value
             else:
-                if self._is_null_value(value):
+                values = (value, )
+
+            for bit in values:
+                if self._is_null_value(bit):
                     continue
 
+                attrs = {'name': key}
+
                 if boost and key in boost:
-                    boost[key] = force_unicode(boost[key])
+                    attrs['boost'] = force_unicode(boost[key])
 
-                    field = ET.Element('field', name=key, boost=boost[key])
-                else:
-                    field = ET.Element('field', name=key)
+                field = ET.Element('field', **attrs)
+                field.text = self._from_python(bit)
 
-                field.text = self._from_python(value)
                 doc_elem.append(field)
 
         return doc_elem
@@ -864,8 +861,8 @@ class Solr(object):
         try:
             # We'll provide the file using its true name as Tika may use that
             # as a file type hint:
-            resp = self._send_request('post', "update/extract",
-                                      headers=params,
+            resp = self._send_request('post', 'update/extract',
+                                      body=params,
                                       files={'file': (file_obj.name, file_obj)})
         except (IOError, SolrError) as err:
             self.log.error("Failed to extract document metadata: %s", err,
